@@ -5,15 +5,19 @@ import pathlib as _pl
 import shutil as _su
 import subprocess as _sp
 import sys as _sys
+import time as _time
 import typing as _tp
 
 import PyQt5.QtWidgets as _qtw
 import pandas as _pd
 import pytest as _pt
+import pytrnsys.utils.log as _ulog
 
-from pytrnsys.utils import log
-import trnsysGUI.GUI as _GUI
+import trnsysGUI.MassFlowVisualizer as _mfv
+import trnsysGUI.TVentil as _tv
+import trnsysGUI.connection.singlePipeConnection as _spc
 import trnsysGUI.diagram.Editor as _de
+import trnsysGUI.mainWindow as _mw
 import trnsysGUI.project as _prj
 import trnsysGUI.storageTank.widget as _stw
 
@@ -36,7 +40,7 @@ class _Project:
 
     @property
     def testId(self) -> str:
-        return f"{self.projectName} [in {self.testCasesDirName}]"
+        return f"{self.testCasesDirName}/{self.projectName}"
 
 
 def getProjects() -> _tp.Iterable[_Project]:
@@ -58,8 +62,9 @@ TEST_CASES = [_pt.param(p, id=p.testId) for p in getProjects()]
 
 class TestEditor:
     @_pt.mark.parametrize("project", TEST_CASES)
-    def testStorageAndHydraulicExports(self, project: _Project,  # pylint: disable=too-many-locals
-                                       request: _pt.FixtureRequest) -> None:
+    def testStorageAndHydraulicExports(  # pylint: disable=too-many-locals
+        self, project: _Project, request: _pt.FixtureRequest
+    ) -> None:
         helper = _Helper(project)
         helper.setup()
 
@@ -78,40 +83,17 @@ class TestEditor:
         mfsDdckRelativePath = f"{project.projectName}_mfs.dck"
         helper.ensureFilesAreEqual(mfsDdckRelativePath)
 
-        editor.exportHydraulics(exportTo="ddck")
-        hydraulicDdckRelativePath = "ddck/hydraulic/hydraulic.ddck"
-        helper.ensureFilesAreEqual(hydraulicDdckRelativePath)
-
         storageTankNames = self._exportStorageTanksAndGetNames(editor)
         for storageTankName in storageTankNames:
             ddckFileRelativePath = f"ddck/{storageTankName}/{storageTankName}.ddck"
             helper.ensureFilesAreEqual(ddckFileRelativePath)
 
-            ddcxFileRelativePath = f"ddck/{storageTankName}/{storageTankName}.ddcx"
-            helper.ensureFilesAreEqual(ddcxFileRelativePath)
+        editor.exportHydraulics(exportTo="ddck")
+        hydraulicDdckRelativePath = "ddck/hydraulic/hydraulic.ddck"
+        helper.ensureFilesAreEqual(hydraulicDdckRelativePath)
 
-        oldFormatJsonPath = projectFolderPath / f"{projectFolderPath.name}.json"
-
-        projectInOldJsonFormat = _prj.LoadProject(oldFormatJsonPath)
-
-        logger = log.setup_custom_logger("root", "DEBUG")  # type: ignore[attr-defined]
-
-        mainWindow = _GUI.MainWindow(logger, projectInOldJsonFormat)  # type: ignore[attr-defined]
-
-        newProjectFolderPath = projectFolderPath.parent / "actual"
-
-        if newProjectFolderPath.exists():
-            _su.rmtree(newProjectFolderPath)
-
-        _os.mkdir(newProjectFolderPath)
-
-        mainWindow.copyContentsToNewFolder(newProjectFolderPath, projectFolderPath)
-
-        newestFormatJsonPath = newProjectFolderPath / f"{newProjectFolderPath.name}.json"
-
-        editor.encodeDiagram(newestFormatJsonPath)
-
-        assert self._createEditor(newProjectFolderPath)
+        editor.exportDdckPlaceHolderValuesJsonFile(shallShowMessageOnSuccess=False)
+        helper.ensureFilesAreEqual("DdckPlaceHolderValues.json")
 
     @classmethod
     def _exportStorageTanksAndGetNames(cls, editor: _de.Editor) -> _tp.Sequence[str]:  # type: ignore[name-defined]
@@ -124,10 +106,43 @@ class TestEditor:
 
         return storageTankNames
 
+    @_pt.mark.parametrize("testProject", TEST_CASES)
+    def testSaveAndReloadProject(  # pylint: disable=too-many-locals
+        self, testProject: _Project, request: _pt.FixtureRequest
+    ) -> None:
+        # The following line is required otherwise QT will crash
+        application = _qtw.QApplication([])
+
+        def quitApplication():
+            application.quit()
+
+        request.addfinalizer(quitApplication)
+
+        logger = _ulog.setup_custom_logger("root", "DEBUG")  # type: ignore[attr-defined]
+
+        helper = _Helper(testProject)
+        helper.setup()
+
+        jsonFilePath = helper.actualProjectFolderPath / f"{helper.actualProjectFolderPath.name}.json"
+        project = _prj.LoadProject(jsonFilePath)
+        mainWindow = _mw.MainWindow(logger, project)  # type: ignore[attr-defined]
+
+        convertedProjectFolderPath = helper.actualProjectFolderPath.parent / "converted"
+        while convertedProjectFolderPath.exists():
+            _su.rmtree(convertedProjectFolderPath)
+            _time.sleep(0.5)
+        _os.mkdir(convertedProjectFolderPath)
+
+        mainWindow.copyContentsToNewFolder(convertedProjectFolderPath, helper.actualProjectFolderPath)
+
+        convertedJsonFilePath = convertedProjectFolderPath / f"{convertedProjectFolderPath.name}.json"
+        convertedProject = _prj.LoadProject(convertedJsonFilePath)
+        _mw.MainWindow(logger, convertedProject)  # type: ignore[attr-defined]
+
     @_pt.mark.linux_ci
-    @_pt.mark.parametrize("project", TEST_CASES)
-    def testMassFlowSolver(self, project: _Project, request: _pt.FixtureRequest) -> None:
-        helper = _Helper(project)
+    @_pt.mark.parametrize("testProject", TEST_CASES)
+    def testMassFlowSolver(self, testProject: _Project, request: _pt.FixtureRequest) -> None:
+        helper = _Helper(testProject)
         helper.setup()
 
         # The following line is required otherwise QT will crash
@@ -139,38 +154,68 @@ class TestEditor:
         request.addfinalizer(quitApplication)
 
         projectFolderPath = helper.actualProjectFolderPath
+        projectJsonFilePath = projectFolderPath / f"{projectFolderPath.name}.json"
+        project = _prj.LoadProject(projectJsonFilePath)
+        logger = _ulog.setup_custom_logger("root", "DEBUG")  # type: ignore[attr-defined]
+        mainWindow = _mw.MainWindow(logger, project)  # type: ignore[attr-defined]
 
-        self._exportMassFlowSolverDeckAndRunTrnsys(projectFolderPath)
+        self._exportMassFlowSolverDeckAndRunTrnsys(mainWindow.centralWidget)
 
-        massFlowRatesPrintFile = f"{project.projectName}_Mfr.prt"
-        helper.ensureCSVsAreEqual(massFlowRatesPrintFile)
+        massFlowRatesPrintFileName = f"{testProject.projectName}_Mfr.prt"
+        helper.ensureCSVsAreEqual(massFlowRatesPrintFileName)
 
-        temperaturesPintFileName = f"{project.projectName}_T.prt"
+        temperaturesPintFileName = f"{testProject.projectName}_T.prt"
         helper.ensureCSVsAreEqual(temperaturesPintFileName)
 
-    def _exportMassFlowSolverDeckAndRunTrnsys(self, projectFolderPath):
-        exportedFilePath = self._exportHydraulic(projectFolderPath, _format="mfs")
+        massFlowRatesPrintFilePath = helper.actualProjectFolderPath / massFlowRatesPrintFileName
+        temperaturesPrintFilePath = helper.actualProjectFolderPath / temperaturesPintFileName
+        self._assertMassFlowVisualizerLoadsData(massFlowRatesPrintFilePath, temperaturesPrintFilePath, mainWindow)
+
+    @staticmethod
+    def _assertMassFlowVisualizerLoadsData(
+        massFlowRatesPrintFilePath: _pl.Path,
+        temperaturesPrintFilePath: _pl.Path,
+        mainWindow: _mw.MainWindow,  # type: ignore[name-defined]
+    ):
+        blockItemsAndConnections = mainWindow.centralWidget.trnsysObj
+        singlePipeConnections = [o for o in blockItemsAndConnections if isinstance(o, _spc.SinglePipeConnection)]
+        valves = [o for o in blockItemsAndConnections if isinstance(o, _tv.TVentil)]
+
+        for singlePipeConnection in singlePipeConnections:
+            firstSegment = singlePipeConnection.firstS
+            assert firstSegment
+
+            firstSegment.labelMass.setPlainText("")
+
+        for valve in valves:
+            valve.posLabel.setPlainText("")
+
+        massFlowSolverVisualizer = _mfv.MassFlowVisualizer(  # type: ignore[attr-defined]  # pylint: disable=unused-variable
+            mainWindow, massFlowRatesPrintFilePath, temperaturesPrintFilePath
+        )
+
+        areAllMassFlowLabelsSet = all(s.firstS and s.firstS.labelMass.toPlainText() for s in singlePipeConnections)
+        areAllValvePositionLabelsSet = all(v.posLabel.toPlainText() for v in valves)
+        assert areAllMassFlowLabelsSet and areAllValvePositionLabelsSet
+
+    def _exportMassFlowSolverDeckAndRunTrnsys(self, editor: _de.Editor):  # type: ignore[name-defined]
+        exportedFilePath = self._exportHydraulic(editor, _format="mfs")
 
         trnsysExePath = _pl.PureWindowsPath(r"C:\TRNSYS18\Exe\TrnExe.exe")
 
         runningOnWindows = _sys.platform.startswith("win32")
         if runningOnWindows:
-            _sp.run(
-                [str(trnsysExePath), str(exportedFilePath), "/H"], check=True
-            )
+            _sp.run([str(trnsysExePath), str(exportedFilePath), "/H"], check=True)
             return
 
         # Running on Linux using Wine
         zDrivePath = _pl.PureWindowsPath("Z:")
         exportedFileWindowswPath = zDrivePath / exportedFilePath
 
-        _sp.run(
-            ["wine", str(trnsysExePath), str(exportedFileWindowswPath), "/H"], check=True
-        )
+        _sp.run(["wine", str(trnsysExePath), str(exportedFileWindowswPath), "/H"], check=True)
 
     @classmethod
-    def _exportHydraulic(cls, projectFolderPath, *, _format) -> str:
-        editor = cls._createEditor(projectFolderPath)
+    def _exportHydraulic(cls, editor: _de.Editor, *, _format) -> str:  # type: ignore[name-defined]
         exportedFilePath = editor.exportHydraulics(exportTo=_format)
         return exportedFilePath
 
@@ -189,8 +234,8 @@ class TestEditor:
 
 class _Helper:
     def __init__(
-            self,
-            project: _Project,
+        self,
+        project: _Project,
     ):
         self._project = project
 
@@ -221,7 +266,7 @@ class _Helper:
     def ensureFilesAreEqual(self, fileRelativePathAsString: str) -> None:
         actualFilePath, expectedFilePath = self._getActualAndExpectedFilePath(fileRelativePathAsString)
         actualContent = actualFilePath.read_text()
-        expectedContent = expectedFilePath.read_text()
+        expectedContent = expectedFilePath.read_text(encoding="windows-1252")
 
         assert actualContent == expectedContent
 
@@ -239,6 +284,11 @@ class _Helper:
 
         assert actualDf.shape == expectedDf.shape
 
+        actualColumns = "\n".join(sorted(actualDf.columns))  # pylint: disable=no-member
+        expectedColumns = "\n".join(sorted(expectedDf.columns))  # pylint: disable=no-member
+
+        assert actualColumns == expectedColumns
+
         maxAbsoluteDifference = (actualDf - expectedDf).max().max()
 
         assert maxAbsoluteDifference <= absoluteTolerance
@@ -248,6 +298,6 @@ class _Helper:
             _su.rmtree(self.actualProjectFolderPath)
 
         pytrnsysGuiDir = _pl.Path(__file__).parents[3]
-        exampleFolderPath = pytrnsysGuiDir / "data" / "examples" / self._project.projectName
+        exampleFolderPath = pytrnsysGuiDir / "data" / "examplesToBeCompleted" / self._project.projectName
 
         _su.copytree(exampleFolderPath, self.actualProjectFolderPath)
