@@ -1,7 +1,6 @@
 # pylint: skip-file
 import dataclasses as _dc
 import logging as _log
-import re
 import typing as _tp
 
 import jinja2 as _jinja
@@ -9,13 +8,18 @@ import jinja2 as _jinja
 import trnsysGUI.TVentil as _tventil
 import trnsysGUI.connection.connectionBase as _cb
 import trnsysGUI.connection.doublePipeConnection as _dpc
+import trnsysGUI.connection.names as _cnames
 import trnsysGUI.connection.singlePipeConnection as _spc
 import trnsysGUI.connection.values as _values
 import trnsysGUI.hydraulicLoops.model as _hlm
-import trnsysGUI.hydraulicLoops.names as _names
-import trnsysGUI.massFlowSolver as _mfs
+import trnsysGUI.hydraulicLoops.names as _lnames
+import trnsysGUI.internalPiping as _ip
+import trnsysGUI.massFlowSolver.export as _mfse
 import trnsysGUI.massFlowSolver.globalNetwork as _gn
+import trnsysGUI.massFlowSolver.names as _mnames
 import trnsysGUI.massFlowSolver.networkModel as _mfn
+import trnsysGUI.temperatures as _temps
+import trnsysGUI.temperatures.hydraulic as _thyd
 
 _UNUSED_INDEX = 0
 
@@ -32,7 +36,7 @@ class Export:
     def __init__(
         self,
         diagramName: str,
-        massFlowContributors: _tp.Sequence[_mfs.MassFlowNetworkContributorMixin],
+        hasInternalPipings: _tp.Sequence[_ip.HasInternalPiping],
         hydraulicLoops: _tp.Sequence[_hlm.HydraulicLoop],
         fluids: _tp.Sequence[_hlm.Fluid],
         logger: _log.Logger,
@@ -41,14 +45,14 @@ class Export:
         self._hydraulicLoops = hydraulicLoops
         self._fluids = fluids
         self._diagramName = diagramName
-        self._massFlowContributors = massFlowContributors
+        self._hasInternalPipings = hasInternalPipings
         self.logger = logger
         self._editor = editor
 
         self.maxChar = 20
 
-        o: _mfs.MassFlowNetworkContributorMixin
-        nodes = [n for c in self._massFlowContributors for n in c.getInternalPiping().openLoopsStartingNodes]
+        o: _ip.HasInternalPiping
+        nodes = [n for c in self._hasInternalPipings for n in c.getInternalPiping().nodes]
         self.lineNumOfPar = len(nodes)
 
         self.numOfPar = 4 * self.lineNumOfPar + 1
@@ -56,10 +60,12 @@ class Export:
     def exportBlackBox(self, exportTo="ddck"):
         f = "*** Black box component temperatures" + "\n"
         equationNr = 0
-        problemEncountered = False
 
-        for t in self._massFlowContributors:
-            status, equations = t.exportBlackBox()
+        for t in self._hasInternalPipings:
+            if not t.shallRenameOutputTemperaturesInHydraulicFile():
+                continue
+
+            equations = _thyd.export(t)
 
             for equation in equations:
                 f += equation + "\n"
@@ -78,78 +84,73 @@ class Export:
         else:
             f = "\nEQUATIONS " + str(equationNr) + "\n" + f + "\n"
 
+        problemEncountered = False
         return problemEncountered, f
 
     def exportDoublePipeParameters(self, exportTo="ddck"):
         doesHydraulicContainDoublePipes = any(
-            [isinstance(obj, _dpc.DoublePipeConnection) for obj in self._massFlowContributors]
+            isinstance(obj, _dpc.DoublePipeConnection) for obj in self._hasInternalPipings
         )
 
         if not doesHydraulicContainDoublePipes:
             return ""
 
-        commentStars = 6 * "*"
+        constantsBlock = """\
+*** Default global PARAMETERS for TYPES 9511 ***
+CONSTANTS 25
 
+****** Pipe and soil properties ******
+dpLength = 579.404 ! Length of buried pipe in m
+dpDiamIn = 0.4028 ! Inner diameter of pipes in m
+dpDiamOut = 0.429 ! Outer diameter of pipes in m
+dpLambda = 175 ! Thermal conductivity of pipe material, kJ/(h*m*K)
+dpDepth = 1.8 ! Buried pipe depth in m
+dpDiamCase = 2 ! Diameter of casing material in m
+dpLambdaFill = 7  ! Thermal conductivity of fill insulation in kJ/hr.m.K
+dpDistPtoP = 0.55  ! Center-to-center pipe spacing in m
+dpLambdaGap = 1.44  ! Thermal conductivity of gap material in kJ/hr.m.K (gravel)
+dpGapThick = 0  ! Gap thickness in m
+
+****** Fluid properties ******
+dpRhoFlu = 1000.0 ! Density of fluid, kg/m^3
+dpLambdaFl = LamWat*3.6  ! Thermal conductivity of fluid in kJ/hr.m.K
+dpCpFl = 4.19 ! Specific heat of fluid, kJ/kg.K
+dpViscFl = 3.078  ! Viscosity of fluid in kg/m.hr
+
+****** Initial conditions ******
+dpTIniHot = 15  ! Initial fluid temperature - pipe 1 in degrees celsius
+dpTIniCold  = 10  ! Initial fluid temperature - pipe 2 in degrees celsius
+
+****** Soil's thermal properties ******
+dpLamdaSl = 8.64  ! Thermal conductivity of soil in kJ/hr.m.K
+dpRhoSl = 1800  ! Density of soil in kg/m^3
+dpCpSl = 1.0  ! Specific heat of soil in kJ/kg.K
+
+****** Definition of nodes ******
+dpNrFlNds = 60  ! Number of fluid nodes
+dpNrSlRad = 10  ! Number of radial soil nodes
+dpSoilThickness = 0.5  ! Thickness of soil around the gravel considered in the model in m
+dpRadNdDist = dpSoilThickness/dpNrSlRad ! Radial distance of any node in m
+dpNrSlAx = 20  ! Number of axial soil nodes
+dpNrSlCirc = 4  ! Number of circumferential soil nodes
+"""
+        massFlowSolverConstantsBlock = """\
+CONSTANTS 3
+TambAvg = 7.96 ! Average surface temperature in degrees celsius
+dTambAmpl = 13.32 ! Amplitude of surface temperature in n degrees celsius
+ddTcwOffset = 36 ! Days of minimum surface temperature
+"""
         if exportTo == "ddck":
-            exportText = "CONSTANTS 25\n"
+            return constantsBlock
         elif exportTo == "mfs":
-            exportText = "CONSTANTS 28\n"
+            return constantsBlock + "\n\n" + massFlowSolverConstantsBlock
         else:
             raise ValueError("Unknown value for `exportTo`", exportTo)
 
-        exportText += "*** Default global PARAMETERS for TYPES 9511" + "\n"
-
-        exportText += commentStars + " pipe and soil properties " + commentStars + "\n"
-        exportText += self._addComment("dpLength = 24.384", "! Length of buried pipe, m")
-        exportText += self._addComment("dpDiamIn = 0.02618", "! Inner diameter of pipes, m")
-        exportText += self._addComment("dpDiamOut = 0.03198", "! Outer diameter of pipes, m")
-        exportText += self._addComment("dpLambda = 1.37067", "! Thermal conductivity of pipe material, kJ/(h*m*K)")
-        exportText += self._addComment("dpDepth = 3.0", "! Buried pipe depth, m")
-        exportText += self._addComment("dpFlowMode = 2", "! Direction of second pipe flow: 1 = same, 2 = opposite")
-        exportText += self._addComment("dpDiamCase = 0.17526", "! Diameter of casing material, m")
-        exportText += self._addComment("dpLambdaFill = 0.14537", "! Thermal conductivity of fill insulation, kJ/(h*m*K)")
-        exportText += self._addComment("dpDistPtoP = 0.06911", "! Center-to-center pipe spacing, m")
-        exportText += self._addComment("dpLambdaGap = 8.722", "! Thermal conductivity of gap material, kJ/(h*m*K)")
-        exportText += self._addComment("dpGapThick = 0.0000", "! Gap thickness, m")
-
-        exportText += commentStars + " fluid properties " + commentStars + "\n"
-        exportText += self._addComment("dpRhoFlu = 1000.0", "! Density of fluid, kg/m^3")
-        exportText += self._addComment("dpLambdaFl = 2.2068", "! Thermal conductivity of fluid, kJ/(h*m*K)")
-        exportText += self._addComment("dpCpFl = 4.19", "! Specific heat of fluid, kJ/(kg*K)")
-        exportText += self._addComment("dpViscFl = 3.078", "! Viscosity of fluid, kg/(m*h)")
-
-        exportText += commentStars + " initial conditions " + commentStars + "\n"
-        exportText += self._addComment("dpTIniHot = 10.0", "! Initial fluid temperature - Pipe hot, deg C")
-        exportText += self._addComment("dpTIniCold = 10.0", "! Initial fluid temperature - Pipe cold, deg C")
-
-        exportText += commentStars + " thermal properties soil " + commentStars + "\n"
-        exportText += self._addComment("dpLamdaSl = 8.722", "! Thermal conductivity of soil, kJ/(h*m*K)")
-        exportText += self._addComment("dpRhoSl = 2500.0", "! Density of soil, kg/m^3")
-        exportText += self._addComment("dpCpSl = 0.84", "! Specific heat of soil, kJ/(kg*K)")
-
-        if exportTo == "mfs":
-            exportText += (
-                commentStars + " general temperature dependency (dependent on weather data) " + commentStars + "\n"
-            )
-            exportText += self._addComment("TambAvg = 7.96", "! Average surface temperature, deg C")
-            exportText += self._addComment("dTambAmpl = 13.32", "! Amplitude of surface temperature, deg C")
-            exportText += self._addComment("ddTcwOffset = 36", "! Days of minimum surface temperature")
-
-        exportText += commentStars + " definition of nodes " + commentStars + "\n"
-        exportText += self._addComment("dpNrFlNds = 100", "! Number of fluid nodes")
-        exportText += self._addComment("dpNrSlRad = 8", "! Number of radial soil nodes")
-        exportText += self._addComment("dpNrSlAx = 10", "! Number of axial soil nodes")
-        exportText += self._addComment("dpNrSlCirc = 4", "! Number of circumferential soil nodes")
-        exportText += self._addComment("dpRadNdDist = 0.0254", "! Radial distance of any node, m")
-
-        exportText += "\n"
-
-        return exportText
-
     def exportPumpOutlets(self):
         f = "*** Pump outlet temperatures" + "\n"
         equationNr = 0
-        for t in self._massFlowContributors:
+        for t in self._hasInternalPipings:
             f += t.exportPumpOutlets()[0]
             equationNr += t.exportPumpOutlets()[1]
 
@@ -164,7 +165,7 @@ class Export:
         f = "*** Massflowrates" + "\n"
         equationNr = 0
 
-        for t in self._massFlowContributors:
+        for t in self._hasInternalPipings:
             f += t.exportMassFlows()[0]
             equationNr += t.exportMassFlows()[1]
 
@@ -185,7 +186,7 @@ class Export:
         nUnit = unit
         constants = 0
         f2 = ""
-        for t in self._massFlowContributors:
+        for t in self._hasInternalPipings:
             f2 += t.exportDivSetting1()[0]
             constants += t.exportDivSetting1()[1]
 
@@ -193,7 +194,7 @@ class Export:
             f = "CONSTANTS " + str(constants) + "\n"
             f += f2 + "\n"
 
-        for t in self._massFlowContributors:
+        for t in self._hasInternalPipings:
             res = t.exportDivSetting2(nUnit)
             f += res[0]
             nUnit = res[1]
@@ -201,20 +202,20 @@ class Export:
         return f
 
     def exportParametersFlowSolver(self, simulationUnit: int, simulationType: int, descConnLength: int) -> str:
-        massFlowContributors = []
-        for massFlowContributor in self._massFlowContributors:
+        hasInternalPipings = []
+        for hasInternalPiping in self._hasInternalPipings:
             noHydraulicConnection = (
-                not isinstance(massFlowContributor, _cb.ConnectionBase)
-                and not massFlowContributor.outputs  # type: ignore[attr-defined]
-                and not massFlowContributor.inputs  # type: ignore[attr-defined]
+                not isinstance(hasInternalPiping, _cb.ConnectionBase)
+                and not hasInternalPiping.outputs  # type: ignore[attr-defined]
+                and not hasInternalPiping.inputs  # type: ignore[attr-defined]
             )
 
             if noHydraulicConnection:
                 continue
 
-            massFlowContributors.append(massFlowContributor)
+            hasInternalPipings.append(hasInternalPiping)
 
-        serializedNodes = self._getSerializedNodes(massFlowContributors)
+        serializedNodes = self._getSerializedNodes(hasInternalPipings)
 
         lines = []
         for serializedNode in serializedNodes:
@@ -241,20 +242,26 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
     @classmethod
     def _getSerializedNodes(
         cls,
-        massFlowContributors: _tp.Sequence[_mfs.MassFlowNetworkContributorMixin],
+        hasInternalPipings: _tp.Sequence[_ip.HasInternalPiping],
     ) -> _tp.Sequence[_SerializedNode]:
-        globalNetwork = _gn.getGlobalNetwork(massFlowContributors)
-        internalPortItemToExternalRealNode = globalNetwork.internalPortItemToExternalRealNode
+        globalNetwork = _gn.getGlobalNetwork(hasInternalPipings)
+        internalPortItemToExternalRealNode = globalNetwork.internalPortItemToExternalNode
 
-        realNodesToIndex = {r: i for i, r in enumerate(globalNetwork.realNodes, start=1)}
+        nodesToIndex = {nwp.node: i for i, nwp in enumerate(globalNetwork.nodesWithParent, start=1)}
 
         serializedNodes = []
-        for index, realNode in enumerate(globalNetwork.realNodes, start=1):
+        for index, nodeWithParent in enumerate(globalNetwork.nodesWithParent, start=1):
+            node = nodeWithParent.node
+            parent = nodeWithParent.parent
+
             neighborsAndUnusedIndexes = cls._getNeighborAndUnusedIndexes(
-                realNode, realNodesToIndex, internalPortItemToExternalRealNode
+                node, nodesToIndex, internalPortItemToExternalRealNode
             )
 
-            serializedNode = _SerializedNode(realNode.name, index, realNode.getNodeType(), neighborsAndUnusedIndexes)
+            displayName = parent.getDisplayName()
+            nodeNameOrEmpty = node.name or ""
+            fullName = f"{displayName}{nodeNameOrEmpty}"
+            serializedNode = _SerializedNode(fullName, index, node.getNodeType(), neighborsAndUnusedIndexes)
 
             serializedNodes.append(serializedNode)
 
@@ -262,13 +269,13 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
 
     @staticmethod
     def _getNeighborAndUnusedIndexes(
-        realNode: _mfn.RealNodeBase,
-        realNodesToIndex: _tp.Mapping[_mfn.RealNodeBase, int],
-        internalPortItemToExternalRealNode: _tp.Mapping[_mfn.PortItem, _mfn.RealNodeBase],
+        realNode: _mfn.Node,
+        realNodesToIndex: _tp.Mapping[_mfn.Node, int],
+        internalPortItemToExternalRealNode: _tp.Mapping[_mfn.PortItem, _mfn.Node],
     ) -> _tp.Tuple[int, int, int]:
         neighborIndexes = []
-        for neighbor in realNode.getNeighbours():
-            if isinstance(neighbor, _mfn.RealNodeBase):
+        for neighbor in realNode.getPortItems():
+            if isinstance(neighbor, _mfn.Node):
                 neighborIndex = realNodesToIndex[neighbor]
             elif isinstance(neighbor, _mfn.PortItem):
                 externalRealNode = internalPortItemToExternalRealNode[neighbor]
@@ -291,70 +298,22 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
 
         return neighborAndUnusedIndexes[0], neighborAndUnusedIndexes[1], neighborAndUnusedIndexes[2]
 
-    def convertToStringList(self, l):
-        res = []
-        temp = ""
-        for i in l:
-            if i == "\n":
-                res.append(temp)
-                temp = ""
-            else:
-                temp += i
-
-        return res
-
-    def findId(self, s):
-        a = s[s.find("!") + 1 : s.find(" ", s.find("!"))]
-        self.logger.debug(a)
-        return a
-
-    def _correctIds(self, lineList):
-        fileCopy = lineList[:]
-        self.logger.debug("fds" + str(fileCopy))
-        fileCopy = [" " + l for l in fileCopy]
-
-        matchNumber = re.compile(r"\d+")
-
-        counter = 0
-        for line in lineList:
-            counter += 1
-            k = self.findId(line)
-            if k != str(counter):
-
-                descConnlen = 15
-                res = fileCopy[:]
-                for l in range(len(fileCopy)):
-                    ids = matchNumber.findall(fileCopy[l])
-                    for i in range(3):
-                        if ids[i] == str(k):
-                            ids[i] = str(counter)
-
-                    fileCopyTempLine = fileCopy[l]
-                    rest = fileCopyTempLine[fileCopyTempLine.find("!") :]
-                    res[l] = ids[0] + " " + ids[1] + " " + ids[2] + " " + ids[3]
-                    res[l] += " " * (descConnlen - len(res[l])) + rest
-
-                fileCopy = res
-                fileCopy = [l.replace("!" + str(k) + " ", "!" + str(counter) + " ") for l in fileCopy]
-
-        return "\n".join(fileCopy)
-
     def exportInputsFlowSolver(self):
         f = ""
-        f += "INPUTS " + str(self.lineNumOfPar) + "! for Type 935\n"
+        f += "INPUTS " + str(self.lineNumOfPar) + "! for Type 9351\n"
 
         numberOfInputs = 0
 
         counter = 0
-        for t in self._massFlowContributors:
-            res = t.exportInputsFlowSolver()
+        for t in self._hasInternalPipings:
+            res = _mfse.exportInputsFlowSolver(t)
             f += res[0]
             counter += res[
                 1
             ]  # DC this is a very strange way to print values, I would like to have 10 values per line and the fact that two go together makes it complicated
             numberOfInputs += res[1]
 
-            if counter > 9 or t == self._massFlowContributors[-1]:
+            if counter > 9 or t == self._hasInternalPipings[-1]:
                 f += "\n"
                 counter = 0
 
@@ -384,13 +343,13 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
 
         tot = ""
 
-        for t in self._massFlowContributors:
+        for t in self._hasInternalPipings:
             noHydraulicConnection = not isinstance(t, _cb.ConnectionBase) and not t.outputs and not t.inputs
 
             if noHydraulicConnection:
                 continue
             else:
-                res = t.exportOutputsFlowSolver(prefix, abc, equationNumber, simulationUnit)
+                res = _mfse.exportOutputsFlowSolver(t, equationNumber, simulationUnit)
                 tot += res[0]
                 equationNumber = res[1]
                 nEqUsed += res[2]
@@ -414,8 +373,7 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
         # typeNr1 = 929 # Temperature calculation from a tee-piece
         # typeNr2 = 931 # Temperature calculation from a pipe
 
-        for t in self._massFlowContributors:
-
+        for t in self._hasInternalPipings:
             res = t.exportPipeAndTeeTypesForTemp(unitNumber)
             f += res[0]
             unitNumber = res[1]
@@ -424,29 +382,104 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
 
         return f
 
-    def exportPrintPipeLosses(self):
-        f = ""
-        lossText = ""
-        rightCounter = 0
+    def exportSinglePipeEnergyBalanceVariables(self):
+        singlePipesWithTrnsysUnits = [
+            ip
+            for ip in self._hasInternalPipings
+            if isinstance(ip, _spc.SinglePipeConnection) and ip.shallCreateTrnsysUnit
+        ]
 
-        for t in self._massFlowContributors:
-            if isinstance(t, _spc.SinglePipeConnection):
-                if rightCounter != 0:
-                    lossText += "+"
-                lossText += "P" + t.displayName + "_kW"
-                rightCounter += 1
-            if isinstance(t, _dpc.DoublePipeConnection):
-                if rightCounter != 0:
-                    lossText += "+"
-                lossText += "P" + t.displayName + "Cold_kW" + "+"
-                lossText += "P" + t.displayName + "Hot_kW"
-                rightCounter += 1
+        if not singlePipesWithTrnsysUnits:
+            return ""
 
-        if rightCounter == 0:
-            lossText += "0"
+        convectedFluxes = [sp.getConvectedHeatFluxVariableName() for sp in singlePipesWithTrnsysUnits]
+        summedConvectedFluxes = "+".join(convectedFluxes)
 
-        f += "*** Pipe losses\nEQUATIONS 1\nPipeLossTot=" + lossText + "\n\n"
-        return f
+        dissipatedFluxes = [sp.getDissipatedHeatFluxVariableName() for sp in singlePipesWithTrnsysUnits]
+        summedDissipatedFluxes = "+".join(dissipatedFluxes)
+
+        internalEnergies = [sp.getInternalHeatVariableName() for sp in singlePipesWithTrnsysUnits]
+        summedInternalEnergies = "+".join(internalEnergies)
+
+        Totals = _cnames.EnergyBalanceTotals.SinglePipe
+        equation = f"""\
+*** Single pipe losses
+EQUATIONS 1
+spPipeEnIntTot = {summedInternalEnergies}
+
+UNIT 100 TYPE 150
+PARAMETERS 3
+1 ! Number of inputs
+1 ! Number of time steps
+0 ! Initial value
+INPUTS 1
+spPipeEnIntTot
+**
+0
+
+EQUATIONS 4
+{Totals.CONVECTED} = {summedConvectedFluxes}
+{Totals.DISSIPATED} = {summedDissipatedFluxes}
+{Totals.PIPE_INTERNAL_CHANGE} = (spPipeEnIntTot - [100,1]) / dtSim / 3600 ! kW
+{Totals.IMBALANCE} = {Totals.CONVECTED} - {Totals.DISSIPATED} - {Totals.PIPE_INTERNAL_CHANGE}
+"""
+        return equation
+
+    def exportDoublePipeEnergyBalanceVariables(self):
+        doublePipes = [ip for ip in self._hasInternalPipings if isinstance(ip, _dpc.DoublePipeConnection)]
+
+        if not doublePipes:
+            return ""
+
+        dissipatedHeatFluxesToFarField = []
+        convectedHeatFluxes = []
+        pipeInternalEnergyChanges = []
+        soilInternalEnergyChanges = []
+        for doublePipe in doublePipes:
+            coldConvectedHeatFlux = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.CONVECTED, _mfn.PortItemType.COLD
+            )
+            hotConvectedHeatFlux = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.CONVECTED, _mfn.PortItemType.HOT
+            )
+            convectedHeatFluxes.extend([coldConvectedHeatFlux, hotConvectedHeatFlux])
+
+            dissipatedHeatFluxToFarField = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.SOIL_TO_FAR_FIELD
+            )
+            dissipatedHeatFluxesToFarField.append(dissipatedHeatFluxToFarField)
+
+            coldPipeInternalEnergyChange = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.PIPE_INTERNAL_CHANGE, _mfn.PortItemType.COLD
+            )
+            hotPipeInternalEnergyChange = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.PIPE_INTERNAL_CHANGE, _mfn.PortItemType.HOT
+            )
+            pipeInternalEnergyChanges.extend([coldPipeInternalEnergyChange, hotPipeInternalEnergyChange])
+
+            soilInternalEnergyChange = doublePipe.getEnergyBalanceVariableName(
+                _dpc.EnergyBalanceVariables.SOIL_INTERNAL_CHANGE
+            )
+            soilInternalEnergyChanges.append(soilInternalEnergyChange)
+
+        summedConvectedHeatFluxes = " + ".join(convectedHeatFluxes)
+        summedDissipationToFarField = " + ".join(dissipatedHeatFluxesToFarField)
+        summedPipeInternalEnergyChanges = " + ".join(pipeInternalEnergyChanges)
+        summedSoilInternalEnergyChanges = " + ".join(soilInternalEnergyChanges)
+
+        Totals = _cnames.EnergyBalanceTotals.DoublePipe
+
+        equations = f"""\
+*** Double pipe energy balance
+EQUATIONS 5
+{Totals.CONVECTED} = {summedConvectedHeatFluxes}
+{Totals.DISSIPATION_TO_FAR_FIELD} = {summedDissipationToFarField}
+{Totals.PIPE_INTERNAL_CHANGE} = {summedPipeInternalEnergyChanges}
+{Totals.SOIL_INTERNAL_CHANGE} = {summedSoilInternalEnergyChanges}
+{Totals.IMBALANCE} = {Totals.CONVECTED} - {Totals.DISSIPATION_TO_FAR_FIELD}  - {Totals.PIPE_INTERNAL_CHANGE} - {Totals.SOIL_INTERNAL_CHANGE}
+"""
+
+        return equations
 
     def exportMassFlowPrinter(self, unitnr, descLen):
         typenr = 25
@@ -484,36 +517,38 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
         f += " " * (descLen - len(f)) + "! 10 Print labels" + "\n"
         f += "\n"
 
-        s = ""
-        equationType = "Mfr"
-        breakline = 0
-        for t in self._massFlowContributors:
-            if isinstance(t, _spc.SinglePipeConnection):
-                breakline, s = self._getEquation(breakline, s, t, "", equationType)
-            if isinstance(t, _dpc.DoublePipeConnection):
-                breakline, s = self._getEquation(breakline, s, t, "Cold", equationType)
-                breakline, s = self._getEquation(breakline, s, t, "Hot", equationType)
-            if isinstance(t, _tventil.TVentil) and t.isVisible():
-                breakline += 1
-                if breakline % 8 == 0:
-                    s += "\n"
-                s += "xFrac" + t.displayName + " "
-        f += "INPUTS " + str(breakline) + "\n" + s + "\n" + "***" + "\n" + s + "\n\n"
+        allVariableNames = []
+        for hasInternalPiping in self._hasInternalPipings:
+            if isinstance(hasInternalPiping, _spc.SinglePipeConnection):
+                mfrVariableName = _mnames.getCanonicalMassFlowVariableName(
+                    hasInternalPiping, hasInternalPiping.modelPipe
+                )
+                allVariableNames.append(mfrVariableName)
+            elif isinstance(hasInternalPiping, _dpc.DoublePipeConnection):
+                coldMfrVariableName = _mnames.getCanonicalMassFlowVariableName(
+                    hasInternalPiping, hasInternalPiping.coldModelPipe
+                )
+                hotMfrVariableName = _mnames.getCanonicalMassFlowVariableName(
+                    hasInternalPiping, hasInternalPiping.hotModelPipe
+                )
+                allVariableNames.extend([coldMfrVariableName, hotMfrVariableName])
+            elif isinstance(hasInternalPiping, _tventil.TVentil):
+                inputVariableName = _mnames.getInputVariableName(hasInternalPiping, hasInternalPiping.modelDiverter)
+                allVariableNames.append(inputVariableName)
 
+        variableNameOctets = [allVariableNames[s : s + 8] for s in range(0, len(allVariableNames), 8)]
+
+        formattedInputVariables = "\n".join(" ".join(o) for o in variableNameOctets) + "\n"
+
+        f += f"""\
+INPUTS {len(allVariableNames)}
+{formattedInputVariables}
+***
+{formattedInputVariables}
+
+"""
         return f
 
-    def _getEquation(self, breakline, s, t, temperature, equationType):
-        breakline += 1
-        if breakline % 8 == 0:
-            s += "\n"
-        s += equationType + t.displayName + temperature + " "
-        return breakline, s
-
-    @staticmethod
-    def _addComment(firstColumn, comment):
-        spacing = 40
-        return str(firstColumn).ljust(spacing) + comment + "\n"
-
     def exportTempPrinter(self, unitnr, descLen):
 
         typenr = 25
@@ -551,17 +586,28 @@ PARAMETERS {len(serializedNodes) * 4 + 1}
         f += " " * (descLen - len(f)) + "! 10 Print labels" + "\n"
         f += "\n"
 
-        s = ""
-        equationType = "T"
-        breakline = 0
-        for t in self._massFlowContributors:
-            if isinstance(t, _spc.SinglePipeConnection):
-                breakline, s = self._getEquation(breakline, s, t, "", equationType)
-            if isinstance(t, _dpc.DoublePipeConnection):
-                breakline, s = self._getEquation(breakline, s, t, "Cold", equationType)
-                breakline, s = self._getEquation(breakline, s, t, "Hot", equationType)
-        f += "INPUTS " + str(breakline) + "\n" + s + "\n" + "***" + "\n" + s + "\n\n"
+        allVariableNames = []
+        for hasInternalPiping in self._hasInternalPipings:
+            if not isinstance(hasInternalPiping, (_spc.SinglePipeConnection, _dpc.DoublePipeConnection)):
+                continue
 
+            internalPiping = hasInternalPiping.getInternalPiping()
+            nodes = internalPiping.nodes
+            temperatureVariableNames = [_temps.getTemperatureVariableName(hasInternalPiping, n) for n in nodes]
+
+            allVariableNames.extend(temperatureVariableNames)
+
+        variableNameOctets = [allVariableNames[s : s + 8] for s in range(0, len(allVariableNames), 8)]
+
+        formattedInputVariables = "\n".join(" ".join(s) for s in variableNameOctets) + "\n"
+
+        f += f"""\
+INPUTS {len(allVariableNames)}
+{formattedInputVariables}
+***
+{formattedInputVariables}
+
+"""
         return f
 
     def exportFluids(self) -> str:
@@ -622,7 +668,7 @@ EQUATIONS {{nEquations}}
 
         nEquations = sum(6 if l.useLoopWideDefaults else 2 for l in loops)
 
-        return self._render(template, loops=loops, nEquations=nEquations, names=_names, values=_values)
+        return self._render(template, loops=loops, nEquations=nEquations, names=_lnames, values=_values)
 
     @staticmethod
     def _render(template: str, /, **kwargs):
