@@ -1,5 +1,4 @@
 # pylint: skip-file
-# type: ignore
 
 import os
 import shutil
@@ -9,15 +8,15 @@ from PyQt5.QtWidgets import QTreeView
 
 import trnsysGUI.createSinglePipePortItem as _cspi
 import trnsysGUI.images as _img
-import trnsysGUI.massFlowSolver as _mfs
+import trnsysGUI.internalPiping
 import trnsysGUI.massFlowSolver.networkModel as _mfn
 from trnsysGUI.BlockItem import BlockItem
-from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel
-from trnsysGUI.MyQTreeView import MyQTreeView
-from trnsysGUI.massFlowSolver import MassFlowNetworkContributorMixin
+from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel  # type: ignore[attr-defined]
+from trnsysGUI.MyQTreeView import MyQTreeView  # type: ignore[attr-defined]
+from trnsysGUI.internalPiping import HasInternalPiping
 
 
-class GroundSourceHx(BlockItem, MassFlowNetworkContributorMixin):
+class GroundSourceHx(BlockItem, HasInternalPiping):
     def __init__(self, trnsysType, parent, **kwargs):
         super(GroundSourceHx, self).__init__(trnsysType, parent, **kwargs)
 
@@ -31,6 +30,9 @@ class GroundSourceHx(BlockItem, MassFlowNetworkContributorMixin):
         self.changeSize()
         self.addTree()
 
+    def getDisplayName(self) -> str:
+        return self.displayName
+
     def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
         return _img.GROUND_SOURCE_HX_SVG
 
@@ -95,16 +97,12 @@ class GroundSourceHx(BlockItem, MassFlowNetworkContributorMixin):
         self.tree.setMinimumHeight(200)
         self.tree.setSortingEnabled(True)
         self.parent.parent().splitter.addWidget(self.tree)
-        
-    def hasDdckPlaceHolders(self):
-        return True
 
     def deleteBlock(self):
         """
         Overridden method to also delete folder
         """
         self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
-        self.deleteConns()
         self.parent.parent().trnsysObj.remove(self)
         self.logger.debug("deleting block " + str(self) + self.displayName)
         self.parent.scene().removeItem(self)
@@ -134,10 +132,9 @@ class GroundSourceHx(BlockItem, MassFlowNetworkContributorMixin):
             self.path = destPath
             self.logger.debug(self.path)
 
-    def getInternalPiping(self) -> _mfs.InternalPiping:
-        inputPort = _mfn.PortItem("input", _mfn.PortItemType.INPUT)
-        outputPort = _mfn.PortItem("output", _mfn.PortItemType.OUTPUT)
+    def getInternalPiping(self) -> trnsysGUI.internalPiping.InternalPiping:
+        inputPort = _mfn.PortItem("In", _mfn.PortItemDirection.INPUT)
+        outputPort = _mfn.PortItem("Out", _mfn.PortItemDirection.OUTPUT)
+        pipe = _mfn.Pipe(inputPort, outputPort)
 
-        pipe = _mfn.Pipe(self.displayName, self.trnsysId, inputPort, outputPort)
-
-        return _mfs.InternalPiping([pipe], {inputPort: self.inputs[0], outputPort: self.outputs[0]})
+        return trnsysGUI.internalPiping.InternalPiping([pipe], {inputPort: self.inputs[0], outputPort: self.outputs[0]})
