@@ -7,24 +7,29 @@ import typing as _tp
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import QMenu, QMessageBox, QTreeView
 
+import trnsysGUI.connection.names as _cnames
 import trnsysGUI.createSinglePipePortItem as _cspi
 import trnsysGUI.images as _img
+import trnsysGUI.internalPiping as _ip
+import trnsysGUI.massFlowSolver.names as _mnames
 import trnsysGUI.massFlowSolver.networkModel as _mfn
 import trnsysGUI.storageTank.model as _model
 import trnsysGUI.storageTank.side as _sd
+import trnsysGUI.temperatures as _temps
 from trnsysGUI import idGenerator as _id
 from trnsysGUI.BlockItem import BlockItem
-from trnsysGUI.HeatExchanger import HeatExchanger  # type: ignore[attr-defined]
 from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel  # type: ignore[attr-defined]
 from trnsysGUI.MyQTreeView import MyQTreeView  # type: ignore[attr-defined]
 from trnsysGUI.directPortPair import DirectPortPair
-from trnsysGUI.massFlowSolver import InternalPiping, MassFlowNetworkContributorMixin
+from trnsysGUI.heatExchanger import HeatExchanger  # type: ignore[attr-defined]
 from trnsysGUI.singlePipePortItem import SinglePipePortItem
 from trnsysGUI.storageTank.ConfigureStorageDialog import ConfigureStorageDialog
 from trnsysGUI.type1924.createType1924 import Type1924_TesPlugFlow  # type: ignore[attr-defined]
+import trnsysGUI.hydraulicLoops.model as _hlm
+import trnsysGUI.hydraulicLoops.names as _lnames
 
 InOut = _tp.Literal["In", "Out"]
-_T = _tp.TypeVar("_T", covariant=True)
+_T_co = _tp.TypeVar("_T_co", covariant=True)
 
 
 @_dc.dataclass
@@ -33,15 +38,17 @@ class PortIds:
     outputId: int
 
 
-class StorageTank(BlockItem,
-                  MassFlowNetworkContributorMixin):
+class StorageTank(BlockItem, _ip.HasInternalPiping):
     # pylint: disable=too-many-instance-attributes,too-many-public-methods
     HEAT_EXCHANGER_WIDTH = 40
 
-    def __init__(self, trnsysType, parent, **kwargs):
+    def __init__(self, trnsysType, parent, **kwargs) -> None:
         super().__init__(trnsysType, parent, **kwargs)
 
         self.parent = parent
+
+        self._hydraulicLoops: _tp.Optional[_hlm.HydraulicLoops] = None
+
         self._idGenerator: _id.IdGenerator = self.parent.parent().idGen
         self.dckFilePath = ""
 
@@ -49,8 +56,6 @@ class StorageTank(BlockItem,
 
         self.heatExchangers: _tp.List[HeatExchanger] = []
 
-        self.blackBoxEquations = []
-
         self.nTes = self.parent.parent().idGen.getStoragenTes()
         self.storageType = self.parent.parent().idGen.getStorageType()
 
@@ -59,6 +64,15 @@ class StorageTank(BlockItem,
         self.path = None
         self.addTree()
 
+    def setHydraulicLoops(self, hydraulicLoops: _hlm.HydraulicLoops) -> None:
+        self._hydraulicLoops = hydraulicLoops
+
+    def getDisplayName(self) -> str:
+        return self.displayName
+
+    def hasDdckPlaceHolders(self) -> bool:
+        return False
+
     @property
     def leftDirectPortPairsPortItems(self):
         return self._getDirectPortPairPortItems(_sd.Side.LEFT)
@@ -68,9 +82,7 @@ class StorageTank(BlockItem,
         return self._getDirectPortPairPortItems(_sd.Side.RIGHT)
 
     def _getDirectPortPairPortItems(self, side: _sd.Side):
-        return [
-            p for dpp in self.directPortPairs if dpp.side == side for p in [dpp.fromPort, dpp.toPort]
-        ]
+        return [p for dpp in self.directPortPairs if dpp.side == side for p in [dpp.fromPort, dpp.toPort]]
 
     def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
         return _img.STORAGE_TANK_SVG
@@ -87,13 +99,13 @@ class StorageTank(BlockItem,
             heatExchanger.parent = self
 
     def addDirectPortPair(  # pylint: disable=too-many-arguments
-            self,
-            trnsysId: int,
-            side: _sd.Side,
-            relativeInputHeight: float,
-            relativeOutputHeight: float,
-            storageTankHeight: float,
-            portIds: _tp.Optional[PortIds] = None,
+        self,
+        trnsysId: int,
+        side: _sd.Side,
+        relativeInputHeight: float,
+        relativeOutputHeight: float,
+        storageTankHeight: float,
+        portIds: _tp.Optional[PortIds] = None,
     ):
         inputPort = self._createPort("i", relativeInputHeight, storageTankHeight, side)
         outputPort = self._createPort("o", relativeOutputHeight, storageTankHeight, side)
@@ -107,16 +119,15 @@ class StorageTank(BlockItem,
             inputPort.id = portIds.inputId
             outputPort.id = portIds.outputId
 
-        directPortPair = DirectPortPair(
-            trnsysId, inputPort, outputPort, relativeInputHeight, relativeOutputHeight, side
-        )
+        directPortPair = DirectPortPair(trnsysId, inputPort, outputPort, relativeInputHeight, relativeOutputHeight, side)
 
         self.directPortPairs.append(directPortPair)
-        self.inputs.append(directPortPair.fromPort)
-        self.outputs.append(directPortPair.toPort)
+
+        self.inputs = [*self.inputs, directPortPair.fromPort]
+        self.outputs = [*self.outputs, directPortPair.toPort]
 
     def _createPort(
-            self, name: str, relativeHeight: float, storageTankHeight: float, side: _sd.Side
+        self, name: str, relativeHeight: float, storageTankHeight: float, side: _sd.Side
     ) -> SinglePipePortItem:
         sideNr = side.toSideNr()
         portItem = _cspi.createSinglePipePortItem(name, sideNr, self)
@@ -150,7 +161,7 @@ class StorageTank(BlockItem,
         super().updateImage()
         self.label.setPos(self.label.pos().x(), self.h)
 
-    def updatePortItemPositions(self, deltaH, deltaW):
+    def updatePortItemPositionsAfterTankSizeChange(self, deltaH, deltaW):
         for portItem in self.inputs + self.outputs:
             oldRelativeHeight = portItem.pos().y() / self.h
             if portItem.side == 0:
@@ -234,12 +245,12 @@ class StorageTank(BlockItem,
         self._decodeInternal(i, offsetX, offsetY, resBlockList, shallSetNamesAndIDs=True)
 
     def _decodeInternal(  # pylint: disable=too-many-arguments
-            self,
-            i,
-            offsetX,
-            offsetY,
-            resBlockList,
-            shallSetNamesAndIDs: bool,
+        self,
+        i,
+        offsetX,
+        offsetY,
+        resBlockList,
+        shallSetNamesAndIDs: bool,
     ):
         self.logger.debug("Loading a Storage in Decoder")
 
@@ -269,8 +280,8 @@ class StorageTank(BlockItem,
         resBlockList.append(self)
 
     def _decodeDirectPortPair(
-            self,
-            portPairModel: _model.DirectPortPair,
+        self,
+        portPairModel: _model.DirectPortPair,
     ) -> None:
         portPair = portPairModel.portPair
 
@@ -292,8 +303,7 @@ class StorageTank(BlockItem,
         name = heatExchangerModel.name + nameSuffix
 
         heatExchanger = self.addHeatExchanger(
-            name, portPair.trnsysId, portPair.side, portPair.inputPort.relativeHeight,
-            portPair.outputPort.relativeHeight
+            name, portPair.trnsysId, portPair.side, portPair.inputPort.relativeHeight, portPair.outputPort.relativeHeight
         )
 
         if shallSetNamesAndIDs:
@@ -303,34 +313,12 @@ class StorageTank(BlockItem,
         heatExchanger.port2.id = portPair.outputPort.id
 
     def decodePaste(  # pylint: disable=too-many-arguments
-            self, i, offset_x, offset_y, resConnList, resBlockList, **kwargs
+        self, i, offset_x, offset_y, resConnList, resBlockList, **kwargs
     ):
         self._decodeInternal(i, offset_x, offset_y, resBlockList, shallSetNamesAndIDs=False)
 
-    def getTemperatureVariableName(self, portItem: SinglePipePortItem) -> str:
-        directPortPair = self._getDirectPortPairForPortItemOrNone(portItem)
-        if directPortPair:
-            return self._getTemperatureVariableNameForDirectPortPairPortItem(directPortPair, portItem)
-
-        heatExchanger = self._getHeatExchangerForPortItem(portItem)
-        if heatExchanger:
-            return self._getTemperatureVariableNameForHeatExchangerPortItem(heatExchanger)
-
-        raise ValueError("Port item doesn't belong to this storage tank.")
-
-    def getFlowSolverParametersId(self, portItem: SinglePipePortItem) -> int:
-        directPortPair = self._getDirectPortPairForPortItemOrNone(portItem)
-        if directPortPair:
-            return directPortPair.trnsysId
-
-        heatExchanger = self._getHeatExchangerForPortItem(portItem)
-        if heatExchanger:
-            return heatExchanger.trnsysId
-
-        raise ValueError("Port item doesn't belong to this storage tank.")
-
     def assignIDsToUninitializedValuesAfterJsonFormatMigration(
-            self, generator: _id.IdGenerator
+        self, generator: _id.IdGenerator
     ) -> None:  # type: ignore[attr-defined]
         for heatExchanger in self.heatExchangers:
             if heatExchanger.trnsysId == generator.UNINITIALIZED_ID:
@@ -353,7 +341,7 @@ class StorageTank(BlockItem,
         return directPortPair
 
     @staticmethod
-    def _getSingleOrNone(iterable: _tp.Iterable[_T]) -> _tp.Optional[_T]:
+    def _getSingleOrNone(iterable: _tp.Iterable[_T_co]) -> _tp.Optional[_T_co]:
         sequence = list(iterable)
 
         if not sequence:
@@ -364,17 +352,6 @@ class StorageTank(BlockItem,
 
         return sequence[0]
 
-    def _getTemperatureVariableNameForDirectPortPairPortItem(self, directPortPair, portItem):
-        isInputPort = directPortPair.fromPort == portItem
-        relativeHeightInPercent = (
-            directPortPair.relativeInputHeightPercent if isInputPort else directPortPair.relativeOutputHeightPercent
-        )
-        return f"T{self.displayName}Port{directPortPair.side.formatDdck()}{relativeHeightInPercent}"
-
-    @staticmethod
-    def _getTemperatureVariableNameForHeatExchangerPortItem(heatExchanger):
-        return f"T{heatExchanger.displayName}"
-
     # Misc
     def contextMenuEvent(self, event):
         menu = QMenu()
@@ -407,87 +384,35 @@ class StorageTank(BlockItem,
     def mouseDoubleClickEvent(self, event):
         ConfigureStorageDialog(self, self.scene().parent())
 
-    # Export related
-    def exportBlackBox(self):
-        equations = []
-        ddcxPath = _os.path.join(self.path, self.displayName)
-        ddcxPath = ddcxPath + ".ddcx"
-        self.exportDck()
-        if _os.path.isfile(ddcxPath):
-            with open(ddcxPath, "r", encoding="windows-1252") as infile:
-                lines = infile.readlines()
-            for line in lines:
-                if line[0] == "T":
-                    equations.append(line.replace("\n", ""))
-            return "success", equations
+    def getInternalPiping(self) -> _ip.InternalPiping:
+        heatExchangerNodes = [hx.modelPipe for hx in self.heatExchangers]
+        heatExchangerPortItems = {
+            mpi: gpi
+            for hx in self.heatExchangers
+            for mpi, gpi in [(hx.modelPipe.fromPort, hx.port1), (hx.modelPipe.toPort, hx.port2)]
+        }
 
-        self.logger.warning("No file at " + ddcxPath)
-        return "noDdckFile", equations
-
-    def getInternalPiping(self) -> InternalPiping:
-        heatExchangerNodes, heatExchangerPortItems = self._createHeatExchangerNodes()
-
-        portPairNodes, portPairsPortItems = self._createPortPairNodes()
+        portPairNodes = [pp.modelPipe for pp in self.directPortPairs]
+        portPairsPortItems = {
+            mpi: gpi
+            for pp in self.directPortPairs
+            for mpi, gpi in [(pp.modelPipe.fromPort, pp.fromPort), (pp.modelPipe.toPort, pp.toPort)]
+        }
 
         nodes = [*heatExchangerNodes, *portPairNodes]
         modelPortItemsToGraphicalPortItem = heatExchangerPortItems | portPairsPortItems
 
-        return InternalPiping(nodes, modelPortItemsToGraphicalPortItem)
+        return _ip.InternalPiping(nodes, modelPortItemsToGraphicalPortItem)
 
-    def _createHeatExchangerNodes(self):
-        heatExchangerPortItems = {}
-        heatExchangerNodes = []
-        for heatExchanger in self.heatExchangers:
-            heatExchangerPortItem1 = _mfn.PortItem("Heat Exchanger Input", _mfn.PortItemType.INPUT)
-            heatExchangerPortItems[heatExchangerPortItem1] = heatExchanger.port1
-
-            heatExchangerPortItem2 = _mfn.PortItem("Heat Exchanger Output", _mfn.PortItemType.OUTPUT)
-            heatExchangerPortItems[heatExchangerPortItem2] = heatExchanger.port2
-
-            name = self._getMassFlowVariableSuffixForHeatExchanger(heatExchanger)
-
-            node = _mfn.Pipe(name, heatExchanger.trnsysId, heatExchangerPortItem1, heatExchangerPortItem2)
-            heatExchangerNodes.append(node)
-
-        return heatExchangerNodes, heatExchangerPortItems
-
-    def _createPortPairNodes(self):
-        portPairsPortItems = {}
-        portPairNodes = []
-        for directPortPair in self.directPortPairs:
-            portPairPortItem1 = _mfn.PortItem("Input", _mfn.PortItemType.INPUT)
-            portPairsPortItems[portPairPortItem1] = directPortPair.fromPort
-
-            portPairPortItem2 = _mfn.PortItem("Output", _mfn.PortItemType.OUTPUT)
-            portPairsPortItems[portPairPortItem2] = directPortPair.toPort
-
-            portPairName = self._getMassFlowVariableSuffixForDirectPortPair(directPortPair)
-
-            node = _mfn.Pipe(portPairName, directPortPair.trnsysId, portPairPortItem1, portPairPortItem2)
-            portPairNodes.append(node)
-
-        return portPairNodes, portPairsPortItems
-
-    def _getMassFlowVariableSuffixForDirectPortPair(self, directPortPair: DirectPortPair):
-        return (
-            f"{self.displayName}Dp{'L' if directPortPair.side.isLeft else 'R'}"
-            f"{directPortPair.relativeInputHeightPercent}-{directPortPair.relativeOutputHeightPercent}"
-        )
-
-    @staticmethod
-    def _getMassFlowVariableSuffixForHeatExchanger(heatExchanger):
-        return heatExchanger.displayName
-
-    def exportDck(self):  # pylint: disable=too-many-locals,too-many-statements
-
-        if not self._checkConnExists():
+    def exportDck(self) -> None:  # pylint: disable=too-many-locals,too-many-statements
+        if not self._areAllPortsConnected():
             msgb = QMessageBox()
             msgb.setText("Please connect all ports before exporting!")
             msgb.exec_()
             return
-        noError = self._debugConn()
+        success = self._debugConn()
 
-        if not noError:
+        if not success:
             qmb = QMessageBox()
             qmb.setText("Ignore connection errors and continue with export?")
             qmb.setStandardButtons(QMessageBox.Save | QMessageBox.Cancel)
@@ -513,63 +438,96 @@ class StorageTank(BlockItem,
             "nHeatSources": 1,
         }
 
-        directPairsPorts = []
-        for directPortPair in self.directPortPairs:
-            incomingConnection = directPortPair.fromPort.connectionList[0]
-            temperatureName = "T" + incomingConnection.displayName
-            massFlowRateName = "Mfr" + incomingConnection.displayName
+        directPairsPorts = self._getDirectPairPortsForExport()
 
-            outgoingConnection = directPortPair.toPort.connectionList[0]
-            reverseTemperatureName = "T" + outgoingConnection.displayName
+        heatExchangerPorts = self._getHeatExchangerPortsForExport()
 
-            inputPos = directPortPair.relativeInputHeight
-            outputPos = directPortPair.relativeOutputHeight
-
-            directPairsPort = {
-                "T": temperatureName,
-                "side": directPortPair.side.formatDdck(),
-                "Mfr": massFlowRateName,
-                "Trev": reverseTemperatureName,
-                "zIn": inputPos,
-                "zOut": outputPos,
-            }
-            directPairsPorts.append(directPairsPort)
-
-        heatExchangerPorts = []
-        for heatExchanger in self.heatExchangers:
-            heatExchangerName = heatExchanger.displayName
-            incomingConnection = heatExchanger.port1.connectionList[0]
-            temperatureName = "T" + incomingConnection.displayName
-            massFlowRateName = "Mfr" + incomingConnection.displayName
-
-            outgoingConnection = heatExchanger.port2.connectionList[0]
-            reverseTemperatureName = "T" + outgoingConnection.displayName
-
-            inputPos = heatExchanger.relativeInputHeight
-            outputPos = heatExchanger.relativeOutputHeight
-
-            heatExchangerPort = {
-                "Name": heatExchangerName,
-                "T": temperatureName,
-                "Mfr": massFlowRateName,
-                "Trev": reverseTemperatureName,
-                "zIn": inputPos,
-                "zOut": outputPos,
-                "cp": "cpwat",
-                "rho": "rhowat",
-            }
-
-            heatExchangerPorts.append(heatExchangerPort)
-
-        auxiliaryPorts = []
-        for _ in range(inputs["nHeatSources"]):
-            dictInputAux = {"zAux": 0.0, "qAux": 0.0}
-            auxiliaryPorts.append(dictInputAux)
+        auxiliaryPorts = self._getAuxiliaryPortForExport(inputs)
 
         tool.setInputs(inputs, directPairsPorts, heatExchangerPorts, auxiliaryPorts)
 
         tool.createDDck(self.path, self.displayName, typeFile="ddck")
 
+    def _getDirectPairPortsForExport(self):
+        directPairsPorts = []
+        for directPortPair in self.directPortPairs:
+            incomingConnection = directPortPair.fromPort.getConnection()
+            inputTemperatureName = _cnames.getTemperatureVariableName(incomingConnection, _mfn.PortItemType.STANDARD)
+
+            modelPipe = directPortPair.modelPipe
+            massFlowRateName = _mnames.getMassFlowVariableName(self, modelPipe, modelPipe.fromPort)
+
+            outgoingConnection = directPortPair.toPort.getConnection()
+            reverseInputTemperatureName = _cnames.getTemperatureVariableName(
+                outgoingConnection, _mfn.PortItemType.STANDARD
+            )
+
+            inputPos = directPortPair.relativeInputHeight
+            outputPos = directPortPair.relativeOutputHeight
+
+            outputTemperatureName = _temps.getInternalTemperatureVariableName(self, modelPipe)
+
+            directPairsPort = {
+                "T": inputTemperatureName,
+                "side": directPortPair.side.formatDdck(),
+                "Mfr": massFlowRateName,
+                "Trev": reverseInputTemperatureName,
+                "zIn": inputPos,
+                "zOut": outputPos,
+                "Tout": outputTemperatureName,
+            }
+            directPairsPorts.append(directPairsPort)
+        return directPairsPorts
+
+    def _getHeatExchangerPortsForExport(self):
+        heatExchangerPorts = []
+        for heatExchanger in self.heatExchangers:
+            heatExchangerPort = self._getHeatExchangerPortForExport(heatExchanger)
+
+            heatExchangerPorts.append(heatExchangerPort)
+        return heatExchangerPorts
+
+    def _getHeatExchangerPortForExport(self, heatExchanger):  # pylint: disable=too-many-locals
+        heatExchangerName = heatExchanger.displayName
+
+        incomingConnection = heatExchanger.port1.getConnection()
+        inputTemperatureName = _cnames.getTemperatureVariableName(incomingConnection, _mfn.PortItemType.STANDARD)
+        modelPipe = heatExchanger.modelPipe
+        massFlowRateName = _mnames.getMassFlowVariableName(self, modelPipe, modelPipe.fromPort)
+
+        outgoingConnection = heatExchanger.port2.getConnection()
+        reverseInputTemperatureName = _cnames.getTemperatureVariableName(outgoingConnection, _mfn.PortItemType.STANDARD)
+
+        inputPos = heatExchanger.relativeInputHeight
+        outputPos = heatExchanger.relativeOutputHeight
+
+        outputTemperatureName = _temps.getInternalTemperatureVariableName(self, modelPipe)
+
+        loop = self._hydraulicLoops.getLoopForExistingConnection(incomingConnection)
+        loopName = loop.name.value
+
+        heatExchangerPort = {
+            "Name": heatExchangerName,
+            "T": inputTemperatureName,
+            "Mfr": massFlowRateName,
+            "Trev": reverseInputTemperatureName,
+            "zIn": inputPos,
+            "zOut": outputPos,
+            "Tout": outputTemperatureName,
+            "cp": _lnames.getHeatCapacityName(loopName),
+            "rho": _lnames.getDensityName(loopName),
+        }
+
+        return heatExchangerPort
+
+    @staticmethod
+    def _getAuxiliaryPortForExport(inputs):
+        auxiliaryPorts = []
+        for _ in range(inputs["nHeatSources"]):
+            dictInputAux = {"zAux": 0.0, "qAux": 0.0}
+            auxiliaryPorts.append(dictInputAux)
+        return auxiliaryPorts
+
     def _debugConn(self):
         self.logger.debug("Debugging conn")
         errorConnList = ""
@@ -595,7 +553,7 @@ class StorageTank(BlockItem,
 
         return noError
 
-    def _checkConnExists(self):
+    def _areAllPortsConnected(self):
         for heatExchanger in self.heatExchangers:
             if not heatExchanger.port1.connectionList:
                 return False
@@ -643,7 +601,6 @@ class StorageTank(BlockItem,
         Overridden method to also delete folder
         """
         self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
-        self.deleteConns()
         self.parent.parent().trnsysObj.remove(self)
         self.logger.debug("deleting block " + str(self) + self.displayName)
         self.parent.scene().removeItem(self)
